<template>
  <!-- 风速设置 -->
  <div class="wind-speed-set wind-box">
    <!-- 标题 -->
    <div class="wind-box-header">
      <!-- 标题图标 -->
      <div class="wind-box-header-icon iconfont iconfont-fengsu" />
      <!-- 标题文本 -->
      <span class="wind-box-header-text" v-text="'风速'" />
    </div>
    <!-- 内容 -->
    <div class="wind-speed-set-content">
      <!-- 特殊风速调节 -->
      <div class="wind-special-set-content" v-if="specialFanData.length">
        <div
          class="wind-special-set-content-box"
          v-for="(spFan, spFanIndex) in specialFanData"
          :key="spFanIndex"
          :class="{ select: spFan.statusName === fanCurrentStatusName, gray: fanHideStatusNameList.includes(spFan.statusName) }"
          @click="fanHandler(spFan.statusName, spFan.name)"
        >
          <i class="iconfont" :class="`iconfont-${spFan.icon.key}`" />
          <span v-text="spFan.name" />
        </div>
      </div>
      <!-- 风速调节 -->
      <div class="wind-speed-set-content-slider">
        <vue-slider ref="slider2" v-model="value2" v-bind="sliderOptions" v-if="sliderOptions.length" @change="onChange">
          <template v-slot:dot="{ value }">
            <div class="custom-dot">
              <i class="iconfont" :class="`iconfont-${sliderOptions.icons[value].key}`" />
              <!-- <gree-icon :name="icons[value - 1]"> -->
            </div>
          </template>
          <template v-slot:process="{ start, end, style }">
            <div
              class="vue-slider-process custom-slider-process"
              :style="[style, { width: `calc(${end - start}% + ${sliderProcess.width})`, left: sliderProcess.left }]"
            />
          </template>
          <template v-slot:step="{ active, value }">
            <div :class="['vue-slider-mark-label', 'custom-label', { active }]">
              <!-- <i class="iconfont" :class="`iconfont-${sliderOptions.icons[value].key}`" /> -->
              <!-- <gree-icon :name="icons[value - 1]"> -->
            </div>
          </template>
        </vue-slider>
      </div>
    </div>
  </div>
</template>

<script>
import { Row } from 'gree-ui';
import { mapGetters } from 'vuex';
import { glyphs } from '@assets/iconfont/iconfont.json';
import { showToast } from '@PluginInterface';
import { throttle } from '@/utils/index';
import VueSlider from 'vue-slider-component';
import 'vue-slider-component/theme/default.css';

export default {
  name: 'wind-speed',
  components: {
    VueSlider,
    [Row.name]: Row
  },
  data() {
    return {
      fanStatusNameList: [], // 风档的顺序
      specialFanStatusNameList: [], // 特殊风档的顺序
      sliderProgress: 60, // 滑条进度
      slderWidth: 0, // 滑动区域
      sliderBarWidth: 0, // 滑条宽度
      sliderToucherWidth: 0, // 滑块宽度
      sliderStart: 0, // 滑条开始滑动位置
      isTouch: false, // 是否滑动中
      value2: 1,
      sliderProcess: {}
    };
  },
  computed: {
    ...mapGetters('machine', ['hideStateNameArr']),
    ...mapGetters(['fanDefine', 'fanIdentifier', 'fanCurrentStatusName', 'fanAbleSet', 'fanLoop']),
    fanData() {
      const result = this.fanStatusNameList.map(fanStatusName => {
        // status定义
        const status = this.fanDefine.statusDefine[fanStatusName];
        // 名称
        const statusNameText = status.name;
        const stateNameText = this.$language(`fan.${this.fanIdentifier}_${statusNameText}`);
        // 图标
        const icon = glyphs.some(item => item.font_class === status.icon.key) ? status.icon : { key: 'undefined', type: 'off' };
        return { name: stateNameText, statusName: fanStatusName, icon };
      });
      return result;
    },
    specialFanData() {
      const result = this.specialFanStatusNameList.map(fanStatusName => {
        // status定义
        const status = this.fanDefine.statusDefine[fanStatusName];
        // 名称
        const statusNameText = status.name;
        const stateNameText = this.$language(`fan.${this.fanIdentifier}_${statusNameText}`);
        // 图标
        const icon = glyphs.some(item => item.font_class === status.icon.key) ? status.icon : { key: 'undefined', type: 'off' };
        return { name: stateNameText, statusName: fanStatusName, icon };
      });
      return result;
    },
    sliderOptions() {
      const { length } = this.fanStatusNameList;
      const marks = {};
      const icons = {};
      const statusNames = {};
      this.fanStatusNameList.forEach((fanStatusName, index) => {
        // status定义
        const status = this.fanDefine.statusDefine[fanStatusName];
        // 名称
        const statusNameText = status.name;
        const stateNameText = this.$language(`fan.${this.fanIdentifier}_${statusNameText}`);
        // 图标
        const icon = glyphs.some(item => item.font_class === status.icon.key) ? status.icon : { key: 'undefined', type: 'off' };
        marks[index + 1] = stateNameText;
        icons[index + 1] = icon;
        statusNames[index + 1] = fanStatusName;
      });
      const result = {
        min: 1,
        max: length || 1,
        interval: 1,
        duration: 0,
        marks,
        icons,
        statusNames,
        length,
        dotOptions: {
          tooltip: 'never'
        },
        dotStyle: {
          'box-shadow': 'none'
        },
        railStyle: {
          'background-color': '#E7EBEB'
        },
        processStyle: {
          background: 'linear-gradient(to right, #8DBFFD , #6CA1E2)'
        },
        tooltipStyle: {
          padding: 0,
          color: '#404657',
          'border-color': '#fff',
          'background-color': '#fff'
        },
        stepStyle: {
          'background-color': '#DADFDD'
        },
        stepActiveStyle: {
          'background-color': '#9EC9FC'
        }
      };
      return result;
    },
    fanCurrentIndex() {
      return this.fanData.findIndex(fan => {
        return fan.statusName === this.fanCurrentStatusName;
      });
    },
    fanDragIndex() {
      // 减去圆所占进度
      const progress = this.fixProgress(this.sliderProgress, true);
      const { sliderStep } = this;
      // 获取正数
      const dec = progress % sliderStep;
      // 获取小数
      const int = Math.floor(progress / sliderStep);
      // 图标下标上限
      const len = this.fanData.legnth - 1;
      // 计算进度条对应的位置
      let index = int + (dec >= sliderStep / 2 ? 1 : 0);
      // 越界处理
      index = index < 0 ? 0 : index;
      index = index > len ? len : index;
      return index;
    },
    fanCurrentInfo() {
      if (!this.fanData || !this.fanData[this.fanDragIndex] || !this.fanData[this.fanDragIndex].icon) {
        return {};
      }
      return this.fanData[this.fanDragIndex];
    },
    sliderStep() {
      return Math.round(100 / (this.fanData.length - 1));
    },
    fanHideStatusNameList() {
      const { fanIdentifier } = this;
      const result = [];
      const reg = new RegExp(`^${fanIdentifier}_`);
      this.hideStateNameArr.forEach(stateName => {
        reg.test(stateName) && result.push(RegExp.rightContext);
      });
      return result;
    },
    throttleShowToast() {
      return throttle(showToast);
    }
  },
  mounted() {
    this.init();
    this.getSliderWidth();
    this.getFanOrder();

    this.sliderProcess.width = Math.floor((window['lib'].flexible.rem / 54) * 66) + 'px';
    this.sliderProcess.left = -Math.floor((window['lib'].flexible.rem / 54) * 33) + 'px';
  },
  watch: {
    fanAbleSet(newVal) {
      newVal || (this.showPopup = false);
    },
    fanLoop() {
      this.init();
    },
    fanCurrentStatusName() {
      this.init();
    },
    fanCurrentIndex: {
      handler(newVal) {
        this.setSliderPos(newVal);
      },
      immediate: true
    }
  },
  methods: {
    init() {
      this.updateFanLoop();
    },
    updateFanLoop() {
      const [specialFanStatusNameList, fanStatusNameList] = this.getFanOrder();
      this.fanStatusNameList = fanStatusNameList;
      this.specialFanStatusNameList = specialFanStatusNameList;
    },
    // 采用自底向上递归方法获取风档排列, 0: 普通风档, 1: 特殊风档
    getFanOrder() {
      const { map } = this.fanDefine;
      const filpMap = {}; // 映射对象
      const endPoints = []; // 指向'undefined'的状态为结束状态，作为递归的起始点

      // 填充映射对象
      for (const key in map) {
        if (Object.hasOwnProperty.call(map, key)) {
          // 获取映射关系
          filpMap[map[key]] = filpMap[map[key]] ? [...filpMap[map[key]], key] : [key];
          // 获取结束状态
          map[key] === 'undefined' && endPoints.push(key);
        }
      }

      // 根据结束状态获取完整状态排列
      const getList = (list, key) => {
        // 排列的起始点
        list.length || list.push(key);
        // 找到指向源并记录
        const points = filpMap[key];
        if (!points || !points.length) return list; // 如果没有指向源，退出
        let point = filpMap[key][0];
        if (point === 'undefined') return list; // 如果指向源为'undefined'状态，退出
        point && !list.includes(point) && list.push(point) && getList(list, point); // 递归表达式
        return list;
      };
      // 根据结束状态向上递归得出状态排列顺序，再反转
      const result = endPoints.map(startPoint => getList([], startPoint).reverse());
      // 包含'default'状态的排列在首位
      result.sort(points => {
        return points.includes('default') ? -1 : 1;
      });

      return result;
    },
    getSliderWidth() {
      try {
        // 屏幕宽度
        const width = document.body.clientWidth;
        // 记录滑条宽度
        this.sliderBarWidth = this.$refs['slider-bar'].clientWidth;
        // 记录滑块宽度
        this.sliderToucherWidth = this.$refs['slider-block'].clientWidth;
        // 记录滑动区域宽度 = 滑条宽度 - 滑块宽度
        this.slderWidth = this.sliderBarWidth - this.sliderToucherWidth;
        //  滑条开始滑动位置 = (屏幕宽度 - 滑动区域宽度) / 2  这里+1是因为大多数情况下需要向上取整
        this.sliderStart = (width - this.slderWidth) / 2 + 1;
      } catch (e) {
        e;
      }
    },
    // 开始拖动滑条
    sliderDragStart(e) {
      // 如果之前没有记录过宽度，则记录
      if (!this.sliderStart) {
        this.getSliderWidth();
      }
      this.sliderDragging(e);
      this.isTouch = true;
    },
    // 滑条拖动中
    sliderDragging(e) {
      const toucher = e.touches[0];
      // 获取手指在屏幕上的位置
      const { clientX } = toucher;
      // 将手指在屏幕上的位置转化为在滑条上的位置
      const dragX = clientX - this.sliderStart;
      // 增加圆所占进度
      const sliderProgress = this.fixProgress((dragX / this.slderWidth) * 100, false);
      // 越界处理
      this.sliderProgress = this.progressOutHandler(sliderProgress);
    },
    // 滑条拖动结束
    sliderDragEnd() {
      let index = this.fanDragIndex;
      // 设置滑块位置
      this.setSliderPos(index);
      // 获取对应model的statusName
      const { statusName, name } = this.fanData[index];
      // 给状态机处理
      this.fanHandler(statusName, name);
      this.isTouch = false;
    },
    // 进度条减去/增加圆环部分; hasRound: 是否包含圆环
    fixProgress(progress, hasRound = true) {
      // 计算圆/半圆所占的进度条
      const fix = (this.sliderToucherWidth / this.sliderBarWidth) * 100;
      // 如果进度条已经包括圆环
      if (hasRound) {
        // 减去圆环所占进度
        return ((progress - fix) / (100 - fix)) * 100;
      }
      // 如果不包括圆环，增加圆环所占进度
      return (progress / 100) * (100 - fix) + fix;
    },
    // 设置滑块位置
    setSliderPos(index) {
      const { sliderStep } = this;
      // 增加圆所占进度
      let val = this.fixProgress(index * sliderStep, false);
      // 越界处理
      this.sliderProgress = this.progressOutHandler(val);
    },
    // 越界处理
    progressOutHandler(progress) {
      let result = progress;
      result = result > 100 ? 100 : result;
      result = result < 0 ? 0 : result;
      return result;
    },
    // 滑条事件
    fanHandler(statusName, name) {
      if (this.fanHideStatusNameList.includes(statusName)) {
        this.$toast(`当前模式下不可选择${name}`);
      } else {
        this.$stateMachine.toStatus(this.fanIdentifier, statusName);
      }
      this.setSliderPos(this.fanCurrentIndex);
    },
    onChange(index) {
      const statusName = this.sliderOptions.statusNames[index];
      const name = this.sliderOptions.marks[index];
      this.fanHandler(statusName, name);
    }
  }
};
</script>

<style lang="scss" scoped>
.wind-box-header {
  margin-bottom: 84px;
}
.wind-speed-set {
  padding: 0 52px;
  &-content {
    $contentWidth: 924px;
    position: relative;
    width: $contentWidth;
    display: flex;
    justify-content: center;
    flex-wrap: wrap;
    &-slider {
      position: relative;
      $sliderHeight: 100px;
      $sliderRadius: 48px;
      // $roundHeight: 68px;
      // $roundMargin: ($sliderHeight - $roundHeight) / 2;
      // background-color: rgba(152, 152, 152, 0.1);
      background-color: rgb(231, 235, 235);
      border-radius: $sliderRadius !important;

      width: 100%;
      // height: $sliderHeight;
      margin-bottom: 36px;
      display: flex;
      justify-content: center;
      margin-bottom: 180px;
      z-index: 1;

      .vue-slider {
        height: $sliderHeight !important;
        width: 800px !important;
        padding: 0 !important;
        .vue-slider-marks {
          border-radius: $sliderRadius !important;
          background-color: #dadfdd;
          &.vue-slider-mark-active {
            background-color: #9ec9fc;
          }
        }
        .vue-slider-rail {
          border-radius: $sliderRadius !important;
          .vue-slider-process {
            border-radius: $sliderRadius !important;
          }
        }
        .vue-slider-dot {
          .custom-dot {
            color: #fff;
            i {
              font-size: 42px;
            }
          }
        }
      }
    }
    &-text {
      width: calc(100% - 30px);
      display: flex;
      flex-wrap: nowrap;
      justify-content: space-between;
      padding: 0 16px;
      font-size: 36px;
      margin-bottom: 74px;
    }
    .wind-special-set-content {
      width: 100%;
      height: 82px;
      display: flex;
      justify-content: space-between;
      align-items: flex-start;
      flex-wrap: nowrap;
      margin-bottom: 60px;
      &-box {
        width: 226px;
        height: 100%;
        background-color: rgba(152, 152, 152, 0.15);
        border-radius: 40px;
        display: flex;
        justify-content: center;
        align-items: center;
        .iconfont {
          font-size: 40px;
          padding-right: 18px;
        }
        span {
          font-size: 42px;
        }
        &.select {
          color: #fff;
          background: linear-gradient(180deg, #8dbffd, rgb(119, 167, 224));
        }
      }
    }
  }
  .gray {
    background-color: transparent;
    color: rgba(64, 70, 87, 0.3);
    &.wind-special-set-content-box {
      background-color: rgba(152, 152, 152, 0.045);
    }
  }
}
</style>
